LoadAddOn('Blizzard_GarrisonUI')
local GMC_G = {}
GMC_G.frame = CreateFrame('FRAME')

GMC_G.currentIndex = 1
GMC_G.timeElapsed = 0
GMC_G.missionReady = false
GMC_G.errorTable = {[1] = 'Succes chance too low.', [2] = 'Not enough followers', [3] = 'Missions did not start.', [4] = 'in party error'}


local function inTable(table, value)
    for k,v in pairs(table) do
        if v == value then
            return true
        end
    end
    return false
end

local function getAvailableFollowers()
    local a = C_Garrison.GetFollowers()
    local b = {}
    
    for i = 1, #a do
       if a[i].isCollected and a[i].status == nil then
          table.insert(b, a[i])
       end
    end

    return b
end

local function getMissionReward(missionID)
    local rewards = C_Garrison.GetMissionRewardInfo(missionID)
    local missionRewards = {}
	for id, reward in pairs(rewards) do
		if reward.itemID then 
            table.insert(missionRewards, 1)
		elseif (reward.followerXP) then
			table.insert(missionRewards, 2)
		else
			if reward.title == 'Money Reward' then
                table.insert(missionRewards, 3)
            elseif reward.title == 'Currency Reward' then
                table.insert(missionRewards, 4)
            end
		end
	end
    return missionRewards
end

local function sortFollowers(a, b)
   local follower1 = a
   local follower2 = b
   
   -- sorting: level > item level > (num counters for mission) > (num traits for mission) > quality > name
   if ( follower1.level ~= follower2.level ) then
      return follower1.level > follower2.level;
   end
   if ( follower1.level == GARRISON_FOLLOWER_MAX_LEVEL and follower1.iLevel ~= follower2.iLevel ) then 
      return follower1.iLevel > follower2.iLevel;
   end
   
   if ( follower1.quality ~= follower2.quality ) then
      return follower1.quality > follower2.quality;
   end
   return strcmputf8i(follower1.name, follower2.name) < 0;
end

local function followerEligible(missionID, follower)
    local rewards = getMissionReward(missionID)
    local status = true
    
    --exp? exlude epic lvl 100 followers
    if inTable(rewards, 2) then
        if follower.level == 100 and follower.quality == 4 and not GMC_G.settings.epicExp then
            status = false
        end
    end
    
    return status
end

local function findCounterFollowers(missionID)
    local followers = C_Garrison.GetBuffedFollowersForMission(missionID)
    local returnT = {}
    
    for guid,_ in pairs(followers) do
        local follower = C_Garrison.GetFollowerInfo(guid)
        if follower.status == nil and followerEligible(missionID, follower) then
            table.insert(returnT, guid)
            --print('found possible counter '..follower.name..' '..follower.followerID)
        end
    end
    
    followers = C_Garrison.GetFollowersTraitsForMission(missionID) --temporary solution for traits
    for guid,_ in pairs(followers) do 
        local follower = C_Garrison.GetFollowerInfo(guid)
        if follower.status == nil and not inTable(returnT, guid) and followerEligible(missionID, follower) then
            table.insert(returnT, guid)
            --print('found possible TRAIT '..follower.name..' '..follower.followerID)
        end
    end
  
    return returnT
end

local function findHighestLevelFollower(missionID)
    local a = getAvailableFollowers()
    table.sort(a, sortFollowers)
    for i = 1, #a do
        if followerEligible(missionID, a[i]) then
            return a[i].followerID
        end
    end
end

local function getAvailableMissions()
    return C_Garrison.GetAvailableMissions()
end

local function getNumAvailableMissions()
    return #getAvailableMissions()-#missionQueue.ignore
end

local function getNumAvailableFollowers()
    --fixthis
    local a = C_Garrison.GetFollowers()
    local count = 0
    
    for i = 1, #a do
       if a[i].isCollected and not a[i].status and followerEligible(GMC_G.aMissions[GMC_G.currentIndex].missionID, a[i]) then
          count = count + 1
       end
    end
    return count
end

local function getSuccesChance(missionID)   
    local _, _, _, successChance = C_Garrison.GetPartyMissionInfo(missionID)
    return successChance
end

local function getDuration(missionID)
    --local totalTimeString, totalTimeSeconds, isMissionTimeImproved, successChance, partyBuffs, isEnvMechanicCountered, xpBonus, materialMultiplier = C_Garrison.GetPartyMissionInfo(missionID)
    local _, totalTimeSeconds = C_Garrison.GetPartyMissionInfo(missionID)
    return (totalTimeSeconds/60)/60
end

local function clickMissionButton(mission)
    GarrisonMissionList_Update();
    PlaySound("UI_Garrison_CommandTable_SelectMission");
    GarrisonMissionFrame.MissionTab.MissionList:Hide();
    GarrisonMissionFrame.MissionTab.MissionPage:Show();
    GarrisonMissionPage_ShowMission(mission);
    GarrisonMissionFrame.followerCounters = C_Garrison.GetBuffedFollowersForMission(mission.missionID)
    GarrisonMissionFrame.followerTraits = C_Garrison.GetFollowersTraitsForMission(mission.missionID);
    GarrisonFollowerList_UpdateFollowers(GarrisonMissionFrame.FollowerList);
end

local function performMission(mission)
    --print('Mission '..mission.missionID..': '..mission.name)
    local addedFollowers = {}
    
    local partySize = mission.numFollowers
    local status = 0


    if getNumAvailableFollowers() < partySize then
        --GarrisonMissionFrame.MissionTab.MissionPage.CloseButton:Click()
        status = 2
        return status
    end
    
    local followers = findCounterFollowers(mission.missionID)
    for i = 1, partySize do
        if followers[i] then
            local follower = C_Garrison.GetFollowerInfo(followers[i])
            --print('Adding '..follower.name..' '..follower.followerID)
            table.insert(addedFollowers, follower.followerID)
            C_Garrison.AddFollowerToMission(mission.missionID, follower.followerID)
        else
            local followerID = findHighestLevelFollower(mission.missionID)
            local a = C_Garrison.GetFollowerInfo(followerID)
            --print('Adding highest '..a.name..' '..followerID)
            table.insert(addedFollowers, followerID)
            C_Garrison.AddFollowerToMission(mission.missionID, followerID)
        end            
    end
    
    if getSuccesChance(mission.missionID) < GMC_G.settings.minimumChance then
        status = 1
    end
    
    if status ~= 0 then
        for i = 1, partySize do
            C_Garrison.RemoveFollowerFromMission(mission.missionID, addedFollowers[i])
        end
    end
    
    return status
end

local function getFirstAvailableMission()
    local missions = getAvailableMissions()
    for k,mission in pairs(missions) do
        if not inTable(missionQueue.ignore, mission.missionID) then
            return mission
        else
            --print('Already in ignore table.')
        end
    end
end

local function createMissionTable()--It's not pretty.
    local allMissions = getAvailableMissions()
    --First get rid of unwanted rewards and missions that are too long
    for k,mission in pairs(allMissions) do
        local missionRewards = getMissionReward(mission.missionID)
        local ignoreCount = 0
        for _, reward in pairs(missionRewards) do
            if inTable(GMC_G.settings.itemIgnoreList, reward) or getDuration(mission.missionID) > GMC_G.settings.maxDuration then
                ignoreCount = ignoreCount + 1
            end            
        end
        if ignoreCount == #missionRewards then
            allMissions[k] = nil
        end
    end

    local finalMissionList = {}
    for i = 1, 3 do
        --Sort table based on priorities.
        for k,mission in pairs(allMissions) do
            local rewards = getMissionReward(mission.missionID)
            local included = false
            for k,rewardType in pairs(rewards) do
                if not included and rewardType == GMC_G.settings.itemPrio[i] then
                    --print('Insert on prio '..mission.name)
                    table.insert(finalMissionList, mission)
                    included = true
                end
            end
            if included then allMissions[k] = nil end
        end
    end
    --Add whatever's left.
    for k,mission in pairs(allMissions) do table.insert(finalMissionList, mission) end
    return finalMissionList  
end



local function missionHandler(self, elapsed)
    GMC_G.timeElapsed = GMC_G.timeElapsed + elapsed
    if GMC_G.timeElapsed >= 0.25 then
        GMC_G.timeElapsed = 0
        
        if GMC_G.currentIndex <= #GMC_G.aMissions and getNumAvailableFollowers() > 0 and not GMC_G.missionReady then
            local mission = GMC_G.aMissions[GMC_G.currentIndex]
            local succes = performMission(mission)

            if succes ~= 0 then
                --print(GMC_G.errorTable[succes])
            else
                GMC_G.missionReady = true
            end

            GMC_G.f.progressText:SetText('Status: '..GMC_G.currentIndex..'/'..#GMC_G.aMissions)
            GMC_G.currentIndex = GMC_G.currentIndex + 1
        elseif GMC_G.missionReady then
            GMC_G.count = GMC_G.count + 1
            C_Garrison.StartMission(GMC_G.aMissions[GMC_G.currentIndex-1].missionID)
            GMC_G.missionReady = false
            PlaySound("UI_Garrison_CommandTable_MissionStart")
        else
            GMC_G.frame:SetScript('OnUpdate', nil)
            print('Garrison Mission Control finished. Started '..GMC_G.count..' mission(s) that match your criteria.')
        end
    end
end

GMC_G.f = {}
local f = GMC_G.f
f.tab = CreateFrame('BUTTON', 'GarrisonMissionFrameTab3', GarrisonMissionFrame, 'GarrisonMissionFrameTabTemplate')
f.tab:SetPoint('CENTER',GarrisonMissionFrameTab2, 'CENTER', 112, 0)
f.tab:SetFrameStrata('BACKGROUND')
f.tab:SetText('GMC Settings')
f.tab:SetID(3)
PanelTemplates_TabResize(f.tab, 10)

PanelTemplates_SetNumTabs(GarrisonMissionFrame, 3)
PanelTemplates_UpdateTabs(GarrisonMissionFrame)

f.mf = CreateFrame('FRAME', 'GMCOptions', GarrisonMissionFrame)
f.mf:SetPoint('CENTER')
f.mf:SetSize(GarrisonMissionFrame:GetWidth(), GarrisonMissionFrame:GetHeight())
f.mf:Hide()
f.header = f.mf:CreateFontString()
f.header:SetFontObject('GameFontNormalHuge')
f.header:SetText('Garrison Mission Control Settings')
f.header:SetPoint('TOP', 0, -48)

local function missionThinker()
    if GMC_G.currentIndex <= #GMC_G.aMissions and getNumAvailableFollowers() > 0 then
        local mission = GMC_G.aMissions[GMC_G.currentIndex]
        local succes = performMission(mission)

        GMC_G.currentIndex = GMC_G.currentIndex + 1
        if succes ~= 0 then
            --print(GMC_G.errorTable[succes])
            missionThinker()
        end
    else
        GMC_G.frame:UnregisterEvent('GARRISON_MISSION_STARTED')
        --print('End of Program')
    end 
end
--[[
GMC_G.frame:SetScript('OnEvent', function()
    print('running through event')
    missionThinker()
end)
--]]
f.startButton = CreateFrame('BUTTON', 'GarrisonMissionControl_Start', GarrisonMissionFrame.MissionTab, 'GameMenuButtonTemplate')
f.startButton:SetText('GMC Start')
f.startButton:SetWidth(148)
f.startButton:SetPoint('BOTTOM', 0, 0)
f.startButton:SetScript('OnClick', function()
    GMC_G.count = 0 
    GMC_G.currentIndex = 1 
    GMC_G.aMissions = createMissionTable() 
    f.progressText:Show() 
    f.progressText:SetText('Status: 1/'..#GMC_G.aMissions) 
    GMC_G.frame:SetScript('OnUpdate', missionHandler) 
end)
f.startButton:SetScript('OnEnter', function() GameTooltip:SetOwner(f.startButton, 'ANCHOR_TOPRIGHT') GameTooltip:AddLine('Assign your followers to missions.') GameTooltip:Show() end)
f.startButton:SetScript('OnLeave', function() GameTooltip:Hide() end)
--f.startButton:SetScript('OnClick', function() GMC_G.currentIndex = 1 GMC_G.aMissions = createMissionTable() GMC_G.frame:RegisterEvent('GARRISON_MISSION_STARTED') missionThinker() end)

f.progressText = f.startButton:CreateFontString()
f.progressText:SetFontObject('GameFontNormal')
f.progressText:SetText('Status: Ready')
f.progressText:SetPoint('RIGHT', 112, 0)
f.progressText:SetTextColor(1, 1, 1)

function GarrisonMissionFrame_SelectTab(id)
   PanelTemplates_SetTab(GarrisonMissionFrame, id);
   if (id == 1) then
      if ( GarrisonMissionFrame.MissionComplete.currentIndex ) then
         GarrisonMissionFrame.MissionComplete:Show();
         GarrisonMissionFrame.MissionCompleteBackground:Show();
         GarrisonMissionFrame.FollowerList:Hide();
      end
      GarrisonMissionFrame.MissionTab:Show();
      GarrisonMissionFrame.FollowerTab:Hide();
      f.mf:Hide()
      f.progressText:SetText('Status: Ready')
      if ( GarrisonMissionFrame.MissionTab.MissionPage:IsShown() ) then
         GarrisonFollowerList_UpdateFollowers(GarrisonMissionFrame.FollowerList);
      end
      GarrisonMissionFrame.TitleText:SetText(GARRISON_MISSIONS_TITLE);
   elseif id == 2 then
      GarrisonMissionFrame.MissionComplete:Hide();
      GarrisonMissionFrame.MissionCompleteBackground:Hide();
      GarrisonMissionFrame.MissionTab:Hide();
      GarrisonMissionFrame.FollowerTab:Show();
      f.mf:Hide()
      if ( GarrisonMissionFrame.FollowerList:IsShown() ) then
         GarrisonFollowerList_UpdateFollowers(GarrisonMissionFrame.FollowerList);
      else
         GarrisonMissionFrame.FollowerList:Show();
      end
      GarrisonMissionFrame.TitleText:SetText(GARRISON_FOLLOWERS_TITLE);
   elseif id == 3 then
      GarrisonMissionFrame.FollowerTab:Hide()
      GarrisonMissionFrame.FollowerList:Hide()
      GarrisonMissionFrame.MissionTab:Hide()
      f.mf:Show()
   end
   if ( UIDropDownMenu_GetCurrentDropDown() == GarrisonFollowerOptionDropDown ) then
      CloseDropDownMenus();
   end
end

--Chance
f.cf = CreateFrame('FRAME', nil, f.mf)
f.cf:SetSize(256, 180)
f.cf:SetPoint('RIGHT', -80, 120)

f.cp = f.cf:CreateTexture(nil, 'BACKGROUND')
f.cp:SetTexture('Interface\\Garrison\\GarrisonMissionUI2.blp')
f.cp:SetAtlas('GarrMission-AllianceChest')
f.cp:SetSize(209-(209*0.25), 155-(155*0.25))
f.cp:SetPoint('CENTER', 0, 0)

f.cc = f.cf:CreateFontString()
f.cc:SetFontObject('GameFontNormalHuge')
f.cc:SetText('Success Chance')
f.cc:SetPoint('TOP', 0, -10)
f.cc:SetTextColor(1, 1, 1)

f.ct = f.cf:CreateFontString()
f.ct:SetFontObject('ZoneTextFont')
f.ct:SetText('50%')
f.ct:SetPoint('CENTER', 0, 0)
f.ct:SetTextColor(0, 1, 0)

f.cs = CreateFrame('Slider', 'GMC_Succes_Slider', f.cf, 'OptionsSliderTemplate')
f.cs:SetWidth(128)
f.cs:SetHeight(20)
f.cs:SetOrientation('HORIZONTAL')
f.cs:SetPoint('BOTTOM', 0, 0)
f.cs:SetMinMaxValues(0, 100)
--f.cs:SetValue(32)
f.cs:SetValueStep(1)
_G[f.cs:GetName() .. 'Low']:SetText('0')       
_G[f.cs:GetName() .. 'High']:SetText('100')    
_G[f.cs:GetName() .. 'Text']:SetText('Minimum chance to start mission.')    
f.cs:SetScript('OnValueChanged', function(self, value)
    local value = math.floor(value)
    f.ct:SetText(value..'%')
    GMC_G.settings.minimumChance = value
end)

--Time
f.tf = CreateFrame('FRAME', nil, f.mf)
f.tf:SetSize(256, 180)
f.tf:SetPoint('LEFT', 80, 120)

f.bg = f.tf:CreateTexture(nil, 'BACKGROUND')
f.bg:SetTexture('Interface\\Timer\\Challenges-Logo.blp')
f.bg:SetSize(100, 100)
f.bg:SetPoint('CENTER', 0, 0)
f.bg:SetBlendMode('ADD')

f.cf = f.tf:CreateTexture(nil, 'BACKGROUND')
--bb:SetTexture('Interface\\Timer\\Challenges-Logo.blp')
--bb:SetTexture('dungeons\\textures\\devices\\mm_clockface_01.blp')
f.cf:SetTexture('World\\Dungeon\\Challenge\\clockRunes.blp')
f.cf:SetSize(110, 110)
f.cf:SetPoint('CENTER', 0, 0)
f.cf:SetBlendMode('ADD')

f.mdt = f.tf:CreateFontString()
f.mdt:SetFontObject('GameFontNormalHuge')
f.mdt:SetText('Mission Duration')
f.mdt:SetPoint('TOP', 0, -10)
f.mdt:SetTextColor(1, 1, 1)

f.mt = f.tf:CreateFontString()
f.mt:SetFontObject('ZoneTextFont')
f.mt:SetText('24h')
f.mt:SetPoint('CENTER', 0, 0)
f.mt:SetTextColor(1, 1, 1)

f.ms = CreateFrame('Slider', 'GMC_Time_Slider', f.tf, 'OptionsSliderTemplate')
f.ms:SetWidth(128)
f.ms:SetHeight(20)
f.ms:SetOrientation('HORIZONTAL')
f.ms:SetPoint('BOTTOM', 0, 0)
f.ms:SetMinMaxValues(1, 24)
--f.ms:SetValue(24)
f.ms:SetValueStep(1)
_G[f.ms:GetName() .. 'Low']:SetText('1')       
_G[f.ms:GetName() .. 'High']:SetText('24')    
_G[f.ms:GetName() .. 'Text']:SetText('Maximum mission duration.')    
f.ms:SetScript('OnValueChanged', function(self, value)
    local value = math.floor(value)
    local c = 1-(value*(1/24))
    if c < 0.3 then c = 0.3 end
    f.mt:SetTextColor(1, c, c)
    f.mt:SetText(value..'h')
    GMC_G.settings.maxDuration = value
end)

--Allowed Items
f.aif = CreateFrame('FRAME', nil, f.mf)
f.aif:SetSize(256, 180)
f.aif:SetPoint('CENTER', 0, 120)

f.itf = f.aif:CreateFontString()
f.itf:SetFontObject('GameFontNormalHuge')
f.itf:SetText('Allowed Rewards')
f.itf:SetPoint('TOP', 0, -10)
f.itf:SetTextColor(1, 1, 1)

f.itf2 = f.aif:CreateFontString()
f.itf2:SetFontObject('GameFontHighlight')
f.itf2:SetText('Click to enable/disable a reward.')
f.itf2:SetPoint('BOTTOM', 0, 0)
f.itf2:SetTextColor(1, 1, 1)


local t = {{t = 'Enable/Disable money rewards.', i = 'Interface\\Icons\\inv_misc_coin_01', key = 3}, 
            {t = 'Enable/Disable other currency awards. (Resources/Seals)', i= 'Interface\\Icons\\inv_garrison_resource', key = 4},
            {t = 'Enable/Disable Follower XP Bonus rewards.', i = 'Interface\\Icons\\XPBonus_Icon', key = 2}, 
            {t = 'Enable/Disable all other regular item rewards.', i = 'Interface\\Icons\\INV_Misc_QuestionMark', key = 1}}
            
GMC_G.f.ignoreFrames = {}
for i = 1, 4 do
    local s = 44
    local x, y = -s, 14
    local x = x + (i-1)*s
    if i == 4 then x = 0 y = -28 end
    local frame = CreateFrame('BUTTON', 'GMC_Allowed_Item'..i, f.aif, 'ItemButtonTemplate')
    frame:SetScale(1.5, 1.5)
    frame:SetPoint('CENTER', x, y)  
    frame.icon:SetTexture(t[i].i)
    
    frame:SetScript('OnClick', function(self)
        if self.icon:IsDesaturated() then
            self.icon:SetDesaturated(false)
            GMC_G.settings.itemIgnoreList[i] = nil
        else
            self.icon:SetDesaturated(true)
             GMC_G.settings.itemIgnoreList[i] = t[i].key
        end
    end)
    
    frame:SetScript('OnEnter', function(self)
        GameTooltip:SetOwner(frame, 'ANCHOR_BOTTOMRIGHT')
        GameTooltip:AddLine(t[i].t);
        GameTooltip:Show()
    end)
    
    frame:SetScript('OnLeave', function() GameTooltip:Hide() end)
    GMC_G.f.ignoreFrames[i] = frame
end

--misc
f.misc = {}
f.misc.f = CreateFrame('FRAME', nil, f.mf)
f.misc.f:SetSize(256, 256)
f.misc.f:SetPoint('RIGHT', -200, -140)

f.misc.t = f.misc.f:CreateFontString()
f.misc.t:SetFontObject('GameFontNormalHuge')
f.misc.t:SetText('Misc. Options')
f.misc.t:SetPoint('TOP', 0, -10)
f.misc.t:SetTextColor(1, 1, 1)

f.misc[1] = f.misc.f:CreateFontString()
f.misc[1]:SetFontObject('GameFontNormal')
f.misc[1]:SetText('Exclude epic lvl 100 followers from exp missions.')
f.misc[1]:SetPoint('TOP', 0, -48)
f.misc[1]:SetTextColor(1, 1, 1)

f.miscb = CreateFrame("CheckButton", nil, f.misc.f, "ChatConfigCheckButtonTemplate")
f.miscb:SetPoint('TOPLEFT', -55, -42)
f.miscb:SetScript('OnClick', function(self) if self:GetChecked() then GMC_G.settings.epicExp = false else GMC_G.settings.epicExp = true end end)

--Prio
f.pf = CreateFrame('FRAME', nil, f.mf)
f.pf:SetSize(256, 180)
f.pf:SetPoint('LEFT', 80, -140)

f.pft = f.pf:CreateFontString()
f.pft:SetFontObject('GameFontNormalHuge')
f.pft:SetText('Item Priority')
f.pft:SetPoint('TOP', 0, -10)
f.pft:SetTextColor(1, 1, 1)

f.pft2 = f.pf:CreateFontString()
f.pft2:SetFontObject('GameFontNormal')
f.pft2:SetText('Prioritize missions with certain a reward.')
f.pft2:SetPoint('BOTTOM', 0, 16)
f.pft2:SetTextColor(1, 1, 1)

local function addPriorityRule(id)
    GMC_G.prioMenu[id].disabled = true
    GMC_G.settings.itemPrio[GMC_G.f.prioFrames.selected] = GMC_G.prioMenu[id].value
    GMC_G.f.prioFrames[GMC_G.f.prioFrames.selected].text:SetText(GMC_G.prioMenu[id].text)
    GMC_G.f.prioFrames[GMC_G.f.prioFrames.selected].abutton:Hide()
    GMC_G.f.prioFrames[GMC_G.f.prioFrames.selected].xbutton:Show()
    GMC_G.f.prioFrames[GMC_G.f.prioFrames.selected].text:Show()
    
    if GMC_G.f.prioFrames.selected ~= 3 then
        GMC_G.f.prioFrames[GMC_G.f.prioFrames.selected+1].f:Show()
    end
end

local function movePrioUp(prio)
    local newPrio = prio-1
    GMC_G.f.prioFrames[newPrio].text:SetText(GMC_G.f.prioFrames[prio].text:GetText())
    GMC_G.f.prioFrames[newPrio].abutton:Hide()
    GMC_G.f.prioFrames[newPrio].xbutton:Show()
    GMC_G.f.prioFrames[newPrio].text:Show()
    GMC_G.settings.itemPrio[newPrio] = GMC_G.settings.itemPrio[prio]
    GMC_G.settings.itemPrio[prio] = 0
    
    GMC_G.f.prioFrames[prio].text:Hide()
    GMC_G.f.prioFrames[prio].xbutton:Hide()
    GMC_G.f.prioFrames[prio].abutton:Show()
end

local function hidePrio(prio)
    GMC_G.f.prioFrames[prio].f:Hide()
end

local function removePriorityRule()
    GMC_G.prioMenu[GMC_G.settings.itemPrio[GMC_G.f.prioFrames.selected]+1].disabled = false
    GMC_G.settings.itemPrio[GMC_G.f.prioFrames.selected] = 0
    GMC_G.f.prioFrames[GMC_G.f.prioFrames.selected].text:Hide()
    GMC_G.f.prioFrames[GMC_G.f.prioFrames.selected].xbutton:Hide()
    GMC_G.f.prioFrames[GMC_G.f.prioFrames.selected].abutton:Show()
    
    if GMC_G.f.prioFrames.selected == 2 then
        if GMC_G.f.prioFrames[3].text:IsShown() then
            movePrioUp(3)
        else
            hidePrio(3)
        end
    elseif GMC_G.f.prioFrames.selected == 1 then
        if GMC_G.f.prioFrames[2].text:IsShown() then
            movePrioUp(2)
            if GMC_G.f.prioFrames[3].text:IsShown() then
                movePrioUp(3)
            else
                hidePrio(3)
            end
        elseif GMC_G.f.prioFrames[3].text:IsShown() then
            movePrioUp(3)
            movePrioUp(2)
        else
            hidePrio(2)
        end  
    end
end
-- 1 = item, 2 = exp, 3 = money, 4 = currency
GMC_G.prioMenu = {
   {text = "Select an item to add as priority.", isTitle = true},
    {text = "Gear/Upgrade Items", func = function() addPriorityRule(2) end, value = 1},
    {text = "Follower XP Reward", func = function() addPriorityRule(3) end, value = 2},
    {text = "Gold Reward", func = function() addPriorityRule(4) end, value = 3},
    {text = "Currency Rewards", func = function() addPriorityRule(5) end, value = 4},
}

f.pmf = CreateFrame("FRAME", "GMC_PRIO_MENU", f.pf, "UIDropDownMenuTemplate")


GMC_G.f.prioFrames = {}
GMC_G.f.prioFrames.selected = 0
for i = 1, 3 do
    GMC_G.f.prioFrames[i] = {}
    local self = GMC_G.f.prioFrames[i]
    self.f = CreateFrame('FRAME', nil, f.pf)
    self.f:SetSize(255, 32)
    self.f:SetPoint('TOP', 0, -38-((i-1)*32))
    if i > 1 then self.f:Hide() end
    
    self.nr = self.f:CreateFontString()
    self.nr:SetFontObject('GameFontNormalHuge')
    self.nr:SetText(i..'.')
    self.nr:SetPoint('LEFT', 8, 0)
    self.nr:SetTextColor(1, 1, 1)
    
    self.text = self.f:CreateFontString()
    self.text:SetFontObject('GameFontNormalLarge')
    self.text:SetText('Def')
    self.text:SetPoint('LEFT', 32, 0)
    --self.text:SetTextColor(1, 1, 0)
    self.text:SetJustifyH('LEFT')
    self.text:Hide()
    
    self.xbutton = CreateFrame('BUTTON', nil, self.f, 'GameMenuButtonTemplate')
    self.xbutton:SetPoint('RIGHT', 0, 0)
    self.xbutton:SetText('X')
    self.xbutton:SetWidth(28)
    self.xbutton:SetScript('OnClick', function() GMC_G.f.prioFrames.selected = i removePriorityRule()  end)
    self.xbutton:Hide()
    
    self.abutton = CreateFrame('BUTTON', nil, self.f, 'GameMenuButtonTemplate')
    self.abutton:SetPoint('CENTER', 0, 0)
    self.abutton:SetText('Add priority rule')
    self.abutton:SetWidth(128)
    self.abutton:SetScript('OnClick', function() GMC_G.f.prioFrames.selected = i EasyMenu(GMC_G.prioMenu, f.pmf, "cursor", 0 , 0, "GMC_G.prioMenu") end)
end

local function updateOptions()
    f.cs:SetValue(GMC_G.settings.minimumChance)
    f.ms:SetValue(GMC_G.settings.maxDuration)
    
    for i = 1, 4 do 
        if GMC_G.settings.itemIgnoreList[i] then 
            f.ignoreFrames[i].icon:SetDesaturated(true) 
        end
    end
    
    for i = 1, 3 do
        if GMC_G.settings.itemPrio[i] ~= 0 then
            --print('rio '..i)
            GMC_G.f.prioFrames[i].f:Show()
            GMC_G.f.prioFrames[i].abutton:Hide()
            GMC_G.f.prioFrames[i].xbutton:Show()
            GMC_G.f.prioFrames[i].text:Show()         
            GMC_G.f.prioFrames[i].text:SetText(GMC_G.prioMenu[GMC_G.settings.itemPrio[i]+1].text)

            if GMC_G.settings.itemPrio[i+1] == 0 and i < 3 then
                GMC_G.f.prioFrames[i+1].f:Show()
            end
        end
    end
    
    if not GMC_G.settings.epicExp then
        f.miscb:SetChecked(true)
    end
end

GMC_G.frame:RegisterEvent('PLAYER_LOGOUT')
GMC_G.frame:RegisterEvent('ADDON_LOADED')
GMC_G.frame:SetScript('OnEvent', function(self, event, arg1) 
	if event == 'ADDON_LOADED' and arg1 == 'GarrisonMissionControl' then
		if GMC_G_Saved then
            GMC_G.settings = GMC_G_Saved
		else
            GMC_G.settings = {}
            GMC_G.settings.itemIgnoreList = {}
            GMC_G.settings.itemPrio = {0, 0, 0}
            GMC_G.settings.minimumChance = 75
            GMC_G.settings.maxDuration = 24
            GMC_G.settings.epicExp = false
		end
        updateOptions()
		--startup stuff after loading vars
	elseif event == 'PLAYER_LOGOUT' then
        GMC_G_Saved = GMC_G.settings
	end
end)